const char *versionstring="2019-09";
/*

   generate periodic lattice from unitcell, 
   (c) GPL Jan02-Sep19, by Joerg Schulenburg
   output as daten.def or povray file

   2019-09-20 JS: add R= ... to out_def for def2plot3d.sh
   2019-03-23 JS: auto-detect dimension from UC, if e1..3 not given
                  also select lattice by name (-lkagome or -l1)
                  some better output 
   2019-03-01 JS: add l6=pyrnn = pyrochlore + 2nd nearest neighbour, +fcc
              JS: fix l3=pyr (use fcc-base)
   2016-05-23 JS: add geometrical anisotrop triangular lattice
   v1.6 Jan13 JS: fix bug + better errors for bigger lattices
   v1.5 Sep09 JS: char *x; x="..."; to const char *x; x="..."
   v1.4 Aug09 JS: output changed from "[+]t0 ..." to "t0 ... [+]"
   v1.3 add uclist of lattices (lattice choosen at runtime now)
        output base = xa instead of "1 0  0 1 ?????"
   v1.1 new daten.def format
   v1.0 initial version 3D, based on pic2.cc, m_sc.c, cubic.c, edgevec.c

   compile: g++ -Wall -o m_lattice m_lattice.c -lm
   use:     ./m_lattice [options] e00 e01 e10 e11
   use:     ./m_lattice [options] e00 e01 e02 e10 e11 e12 e20 e21 e22
   options:
     -t<n>   output type, 0=def, 1=povray, 2=gnuplot3d (default=0)
     -d<n>   lattice dimension (not needed)
     -a      use a cubic base (ignore definition)
     -l[<n>] lattice number (see below)
   examples:
     ./m_lattice     2 1  1 2	        # volume=Det((2,1),(1,2))=3
     ./m_lattice     2 0  0 2 >a.def    # volume=4
     ./m_lattice -t1 2 0  0 2 >a.pov
     ./m_lattice -l3 -t2 2 0 0  0 2 0  0 0 2 >a.plt  # 3D-gnuplot
     gnuplot a.plt # 3D-image can be rotated using mouse
     nice -30 povray +FPPM +L/usr/lib/povray31/include\
       +L/usr/lib/povray3/include +W500 +H375 +K0 +Oa.ppm +Ia.pov +A
     # ppmtopgm a.ppm | 
     pnmtops -rle -noturn a.ppm >a.eps
  
  ToDo:
    - output FCC as 4_per_SC  und  1_per_FCC-UC, same SP-results?
    - read unitcell from file, or table inc. name-string like m_1d.c
    - animation of rotating or translations (til4->kago->til9 ...)
    - wire frame 3D viewer (Drahtgitter 3D Anzeige, => magnon.tcl)
    - print only some layers (example: [1,-1,1] layer)
      and the rest half transparent (opaque) => video?
    - rasmol support? 4MB libcbf0+libvector2 (3D-shaded molecules, XYZ-files)
    - output daten.sym from UC-geometry

  remarks:   
   edgevectors mcf:  1,0,0,1,  1,1,1,-1,  2,1,1,2,  2,0,0,2,  2,1,-1,2,  
               utlf: 1,0,0,1,             3,0,2,1,  2,0,0,2,
               (most compact form, upper triangular lattice form)

   read unitcell from file

 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h> /* 2019-03 add strstr search name */

#define Layer1m11  /* print only [1,-1,1] layer (rgbt t=transparency) */

typedef struct {
  int type;      /* type of site (0=data-stop) */
  double pos[3]; /* coordinates of the site (ToDo: obsolete? => vec) */
} site_t;

/* type s1 s2 rel_unitcell[3] bond_direction[3] phaseindex */
typedef struct {
  int type;      /* type of bond (0=data-stop) ToDo: "J1 t2" */
  int s1,s2;     /* bonded sites */
  int r2[3];     /* relativ coords of site 2 in unit cells */
  double vec[3]; /* direction of bonds ((pos[s2]+u2*a)-pos[s1]) */
  int w;         /* phase index (magnetic fields) = vec-idx */
} bond_t;

/*
   square lattices: x^2+y^2=V            V=1,2,4,5,8,9,10,...
   cubic lattices: (x^2+y^2+z^2)^3=V^2   V=1,64,...
*/
/* --- example: squared unitcell with 4 sites and 6 bonds ---
      +-----------------+
      |        |        |
      |        0        |      ^ a1=(0,1)   xa=base-vectors_of_Unit_Cell_UC
      |      /   \      |      |            xa=(e1,e2,e3)
      |--- 1       2 ---|      |
      |      \   /      |      |
      |        3        |      |
      |        |        |      --------> a0=(1,0)
      +-----------------+
*/
 double til11_xa[3][3]={{1,0,0},{0,1,0},{0,0,1}}; /* base vectors of UC */
 site_t til11_xs[]={
  { 1, {0.5 ,0.75,0}}, /* site 0 */
  { 1, {0.25,0.5 ,0}}, /* site 1 */
  { 1, {0.75,0.5 ,0}}, /* site 2 */
  { 1, {0.5 ,0.25,0}}, /* site 3 */
  { 0, {0   ,0   ,0}}};
 bond_t til11_xb[]={
 /* type s0 s1  r2[3]    vec[3]=vec[s1-s0] w */ 
  { 1,  0, 1, { 0, 0, 0}, {-0.25,-0.25,0}, 0}, /* intra coupling */
  { 1,  0, 2, { 0, 0, 0}, { 0.25,-0.25,0}, 1},
  { 1,  1, 3, { 0, 0, 0}, { 0.25,-0.25,0}, 1},
  { 1,  2, 3, { 0, 0, 0}, {-0.25,-0.25,0}, 0},
  { 2,  0, 3, { 0, 1, 0}, { 0   , 0.5 ,0}, 2}, /* inter coupling */
  { 2,  2, 1, { 1, 0, 0}, { 0.5 , 0   ,0}, 3},
  { 0,  0, 0, { 0, 0, 0}, { 0   , 0   ,0}, 0}}; /* EOT */

#define q3 1.732050808  /* suare-root of 3 */
/* --- kagome unitcell with 3 sites and 6 bonds ---
      +-------1-------+
      |       | \     |
      |       |   \   |      ^ a1=(cos(60)=1/2,sin(60)=0.866=q3/2)
      |       |     \ |      |
      2 ----- 0 ----- 2      |
      | \     |       |      |
      |   \   |       |      |
      |     \ |       |      O-------> a0=(1,0)
      +-------1-------+
*/
 double til8_xa[3][3]={{1,0,0},{0.5,0.5*q3,0},{0,0,1}}; /* base vectors of unitcell */
 site_t til8_xs[]={
  { 1, { 0.75, 0.75, 0}}, /* site 0 shifted by (1/4,1/4,0) = centered */
  { 1, { 0.75, 0.25, 0}}, /* site 1 */
  { 1, { 0.25, 0.75, 0}}, /* site 2 */
  { 0, {0   ,0   ,0}}};
 bond_t til8_xb[]={
 /* type s0 s1  r2[3]     vec[3]=R[s1-s0]  w */ 
  { 1,  1, 0, { 0, 0, 0}, { 0   , 0.5 ,0}, 0}, /*  60deg */
  { 1,  0, 1, { 0, 1, 0}, { 0   , 0.5 ,0}, 0},
  { 1,  2, 0, { 0, 0, 0}, { 0.5 , 0   ,0}, 1}, /*   0deg */
  { 1,  0, 2, { 1, 0, 0}, { 0.5 , 0   ,0}, 1},
  { 1,  1, 2, { 0, 0, 0}, {-0.5 , 0.5 ,0}, 2}, /* 120deg */
  { 1,  2, 1, {-1, 1, 0}, {-0.5 , 0.5 ,0}, 2},
  { 0,  0, 0, {0,0,0}, { 0   , 0   ,0}, 0}};

/* --- triangle anisotrop 1 sites and 3 bonds ---
      0-------0-------0
      | \     | \     |
      |   \   |   \   |        a1=(cos(60)=1/2,sin(60)=0.866=q3/2)
      |     \ |     \ |      ^
      0 ----- 0 ----- 0      |
      | \     | \     |      |
      |   \   |   \   |      |
      |     \ |     \ |      O-------> a0=(1,0)
      0-------0-------0
*/
 double til1_xa[3][3]={{1,0,0},{0.5,0.5*q3,0},{0,0,1}}; /* base vectors of unitcell */
 site_t til1_xs[]={
  { 1, {0.5 ,0.5 ,0}}, /* site 0 */
  { 0, {0   ,0   ,0}}};
 bond_t til1_xb[]={
/* type s1 s2 rel_unitcell[3] bond_direction[3] phaseindex */
  { 1,  0, 0, { 0, 1, 0}, { 0 , 1 ,0}, 0}, /*  60deg */
  { 2,  0, 0, { 1, 0, 0}, { 1 , 0 ,0}, 0}, /*   0deg */
  { 1,  0, 0, {-1, 1, 0}, {-1 , 1 ,0}, 2}, /* 120deg */
  { 0,  0, 0, {0,0,0}, { 0   , 0   ,0}, 0}};

/* --- stretched kagome unitcell with 9 sites and 12+6 bonds ---
      +--------1--------+
      |        | \      |
      |        3---5    |      ^ a1=(cos(60)=1/2,sin(60)=0.866=q3/2)
      |        | \ | \  |      |
      2 --7--- 0 --4--- 2      |
      | \ |  \ |        |      |
      |   8----6        |      |
      |     \  |        |      --------> a0=(1,0)
      +--------1--------+
*/
 /* base coordintes, where all following vectors based on */
 double ska_xa[3][3]={{1,0,0},{0.5,0.5*q3,0},{0,0,1}}; /* base vectors of unitcell */
 site_t ska_xs[]={
 /* type pos=x,y,z */
  { 1, {0.5 ,0.5 ,0}}, /* site 0 */
  { 1, {0.5 ,0   ,0}}, /* site 1 */
  { 1, {0   ,0.5 ,0}}, /* site 2 */
  { 2, {0.5 ,0.75,0}}, /* site 3 */
  { 2, {0.75,0.5 ,0}}, /* site 4 */
  { 2, {0.75,0.75,0}}, /* site 5 */
  { 2, {0.5 ,0.25,0}}, /* site 6 */
  { 2, {0.25,0.5 ,0}}, /* site 7 */
  { 2, {0.25,0.25,0}}, /* site 8 */
  { 0, {0   ,0   ,0}}}; /* End Marker */
 bond_t ska_xb[]={
 /* type s1 s2  Ruc2-Ruc1   Rs2-Rs1 (uc=unit_cell) */
  { 1,  1, 6, { 0, 0, 0}, { 0   , 0.25,0}, 0}, /*  60deg */
  { 1,  6, 0, { 0, 0, 0}, { 0   , 0.25,0}, 0},
  { 1,  0, 3, { 0, 0, 0}, { 0   , 0.25,0}, 0},
  { 1,  3, 1, { 0, 1, 0}, { 0   , 0.25,0}, 0},
  { 1,  2, 7, { 0, 0, 0}, { 0.25, 0   ,0}, 1}, /*   0deg */
  { 1,  7, 0, { 0, 0, 0}, { 0.25, 0   ,0}, 1},
  { 1,  0, 4, { 0, 0, 0}, { 0.25, 0   ,0}, 1},
  { 1,  4, 2, { 1, 0, 0}, { 0.25, 0   ,0}, 1},
  { 1,  1, 8, { 0, 0, 0}, {-0.25, 0.25,0}, 2}, /* 120deg */
  { 1,  8, 2, { 0, 0, 0}, {-0.25, 0.25,0}, 2},
  { 1,  2, 5, {-1, 0, 0}, {-0.25, 0.25,0}, 2},
  { 1,  5, 1, { 0, 1, 0}, {-0.25, 0.25,0}, 2},
  { 2,  4, 5, { 0, 0, 0}, { 0   , 0.25,0}, 0}, /*  60deg */
  { 2,  8, 7, { 0, 0, 0}, { 0   , 0.25,0}, 0},
  { 2,  3, 5, { 0, 0, 0}, { 0.25, 0   ,0}, 1}, /*   0deg */
  { 2,  8, 6, { 0, 0, 0}, { 0.25, 0   ,0}, 1},
  { 2,  4, 3, { 0, 0, 0}, {-0.25, 0.25,0}, 2}, /* 120deg */
  { 2,  6, 7, { 0, 0, 0}, {-0.25, 0.25,0}, 2},
  { 0,  0, 0, {0,0,0}, { 0   , 0   ,0}, 0}};

/* --- cm0103231 Fig 1: 6/2=3 bonds/site
  two interpenetrating pyrochlore sublattices with half unitcell
  cubic unit cell 2x2x2 N=4 Nb=3*4=2*6=12 (similar to BCC-lattice)
  two tetrahedra per 2x2x2 cube, alternating J1-J2-tetrahedra
  tetraeder lying in two diagonal 1x1x1 cubes, corner sharing
  4 diagonal corners of the 1x1x1 cube build the corners of one tetraeder
  Ihle: Magnetit, 860K-ferrimagnet Fe3O4=Fe(3+)(Fe(2+)Fe(3+))O(2-)4
     the oxygen atoms sit on the other 4 corners of the two 1x1x1 cubes
     Fe(2+) cary s=1/2, Hubbardmodel with AF-nn and nnn
  to get an overview tinker some tetraeder using paper and glue together ;)
  4x4x4 has 256 sites: spinpack-2.24.searchT0 > 14days/P2GHz
*/
#if 0 /* not 100% correct, works in most but not all cases 2019-03 */
/*
   incorrectness checked: see below, only 7 of 11 utlf vectors correct
   view from each side (x,y,z) it looks like 2D-checkerboard, correct?
   view from (0,1,1) 2D triangle rombus pattern
   view from (1,1,1)? 2D overlapping tetraeder from top
      = corner-sharing stars (J2 tetras 60deg above J1-tetras)
 test1903.l3: 2*2*2 cube n=32 nb=3*n j2=j1
//   l3: nud=4+28 k=0 sym=0/1416++ n1= 35960 E0= 8.00 8.38196601
//   l6: nud=4+28 k=0 sym=0/1416++ n1= 35960 E0= 8.00 8.04029686
// set view  0,0 # l3 = checker board projection
// set view 56,0 # l3 = squares + 2nd diag
// no hexagons between 3 A + 3 B tetraeder (like pyrnn), screws instead
//  old pyr_-definitions are distorted to SC but seems to
//    show correct topology (bad distorted geometry) HTE12 pyr == pyrnn
// # old l3 lattice 2019-02 can show some, but not all l6-PBCs
// -l3 2 1 0  0 2 0  1 0 2   nn  8*4 4+28 n1 35960 E0 8.0 8.01925771 = l6
// -l3 8 0 0  3 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.03554922 = l6
// -l3 8 0 0  2 1 0  1 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.03603449 = l6
// -l3 2 0 0  0 2 0  0 1 2   nn  8*4 4+28 n1 35960 E0 8.0 8.04029686 = l6
// -l3 8 0 0  2 1 0  2 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.07607561 = l6
// -l3 8 0 0  0 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.08101405 = l6
// -l3 8 0 0  1 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.11489599 = l6
// -l3 8 0 0  1 1 0  1 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.15846977 miss l6
// -l3 8 0 0  3 1 0  1 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.19806226 miss l6
// -l3 4  0 0  1 1 0   0 0 2 nn  8*4 4+28 n1 35960 E0 8.0 8.26903069 miss l6
// -l3 2  0 0  0 2 0   0 0 2 nn  8*4 4+28 n1 35960 E0 8.0 8.38196601 miss l6
//   whats wrong with l3? try kago in square base?
// l6 view0,0 checkerboard
// l3 view0,0 checkerboard
// l3 gleiche innere Topologie, falsche Rand-Topologie !?  2019-02
// l3 view20,45 verzerrtes Sechseck durch shift 1 -1 0  2 2 0  -1 1 2
// l3 view90,45 zeigt shift z=+1
// l3 view60,45 zeigt kagome-proj. jede zweite Ebene shifted
// l3 view140,45 zeigt stacked kagome-ebenen
// ToDo:  compare 2D kagome in square and hexagon base
*/
 double pyr_xa[3][3]={{1,0,0},{0,1,0},{0,0,1}}; /* wrong? SC */
// double pyr_xa[3][3]={{0,1,1},{1,0,1},{1,1,0}}; /* 2019-03 ? */
// next site 0 at (1,0,0)/1 by shift 1*e1 but l9 at (2,0,0)
 site_t pyr_xs[]={
  { 1, {0   ,0   ,0  }}, /* site 0 type=1 posXYZ=0,0,0    */
  { 1, {0.5 ,0.5 ,0  }}, /* site 1 type=1 posXYZ=1,1,0 /2 */
  { 1, {0.5 ,0   ,0.5}}, /* site 2 type=1 posXYZ=1,0,1 /2 */
  { 1, {0   ,0.5 ,0.5}}, /* site 3 type=1 posXYZ=0,1,1 /2 */
  { 0, {0   ,0   ,0}}};  /* end list type=0 */
 bond_t pyr_xb[]={ /* projection: checkerboard (0,0,1), square+diag/2 */
 /* bondtype, site1, site2, UnitCell-shiftXYZ, dXYZ phaseindex=N/A */
  { 1,  0, 1, { 0, 0, 0}, { 0.5 , 0.5 , 0  }, 0}, /* tetraeder 1 */
  { 1,  0, 2, { 0, 0, 0}, { 0.5 , 0   , 0.5}, 0}, /* phasefactor n.a. */
  { 1,  0, 3, { 0, 0, 0}, { 0   , 0.5 , 0.5}, 0},
  { 1,  1, 2, { 0, 0, 0}, { 0   ,-0.5 , 0.5}, 0},
  { 1,  1, 3, { 0, 0, 0}, {-0.5 , 0   , 0.5}, 0},
  { 1,  2, 3, { 0, 0, 0}, {-0.5 , 0.5 , 0  }, 0},
  { 2,  0, 1, {-1,-1, 0}, {-0.5 ,-0.5 , 0  }, 0}, /* tetraeder 2 */
  { 2,  0, 2, {-1, 0,-1}, {-0.5 , 0   ,-0.5}, 0}, /* wrong vecs ? */
  { 2,  0, 3, { 0,-1,-1}, { 0   ,-0.5 ,-0.5}, 0},
  { 2,  1, 2, { 0, 1,-1}, { 0   , 0.5 ,-0.5}, 0},
  { 2,  1, 3, { 1, 0,-1}, { 0.5 , 0   ,-0.5}, 0},
  { 2,  2, 3, { 1,-1, 0}, { 0.5 ,-0.5 , 0  }, 0},  /* 1..12 */
  { 0,  0, 0, { 0, 0, 0}, { 0   , 0   , 0  }, 0}}; /* ToDo: syms? */
#else /* 2019-03-13 this FCC defined geometry works always */
 double pyr_xa[3][3]={{1,1,0},{-1,1,0},{1,0,1}}; /* 2019-03 FCC-UC e1..e3 */
 site_t pyr_xs[]={ /* x=right y=away z=up */
/* s#   *e1  *e2  *e3  (in xa[]=UC-base coordinates) */
  { 1, { 0  , 0  , 0  }}, /* site 0 type=1 posXYZ=0,0,0    */
  { 1, { 0  ,-0.5, 0  }}, /* site 1 type=1 posXYZ=1,-1,0 /2 */
  { 1, { 0  , 0  , 0.5}}, /* site 2 type=1 posXYZ=1,0,1 /2 */
  { 1, {-0.5, 0  , 0.5}}, /* site 3 type=1 posXYZ=0,1,1 /2 */
  { 0, { 0  , 0  , 0  }}};  /* end list type=0, syms=... */
 bond_t pyr_xb[]={ /* projections: kagome (0,1,1), checkerboard (0,0,1) */
 /* bondtype, site1, site2, UnitCell-shiftXYZ, dXYZ phaseindex=N/A */
 /* type s0 s1  r2[3]      vec[3]_xa-units w  sorted by s1<s2 */ 
// gnuplot.splot # 3D-to-2D-projection:
// set view  0.0,0.0  # proj. J1=checker J2=asym_star_inlet 2019-03 buggy?
// set view 56.0,0.0  # proj. J1=kagome J2=asym_kago_inlet  buggyJ2 ?
// set view 90.0,42.0 # proj. J1=kagome (from 1,1,0) */
  { 1,  0, 1, { 0, 0, 0}, { 0  ,-0.5, 0  }, 0}, /* 0-1 tetraeder1 */
  { 1,  0, 2, { 0, 0, 0}, { 0  , 0  , 0.5}, 0}, /* 0-2 phasefactor n.a. */
  { 1,  0, 3, { 0, 0, 0}, {-0.5, 0  , 0.5}, 0}, /* 0-3 */
  { 1,  1, 2, { 0, 0, 0}, { 0  , 0.5, 0.5}, 0}, /* 1-0 + 0-2 = 1-2 */
  { 1,  1, 3, { 0, 0, 0}, {-0.5, 0.5, 0.5}, 0}, /* 1-0+0-2+2-3=1-3 */
  { 1,  2, 3, { 0, 0, 0}, {-0.5, 0  , 0  }, 0}, /* 2-3 */
  { 2,  0, 1, { 0, 1, 0}, { 0  , 0.5, 0  }, 0}, /* 0-1' tetraeder2 */
  { 2,  0, 2, { 0, 0,-1}, { 0  , 0  ,-0.5}, 0}, /* 0-2' */
  { 2,  0, 3, { 1, 0,-1}, { 0.5, 0  ,-0.5}, 0}, /* 0-3'=0-2+2-3 */
  { 2,  1, 2, { 0,-1,-1}, { 0  ,-0.5,-0.5}, 0}, /* 1-2' */
  { 2,  1, 3, { 1,-1,-1}, { 0.5,-0.5,-0.5}, 0}, /* 1-3' */
  { 2,  2, 3, { 1, 0, 0}, { 0.5, 0  , 0  }, 0}, /* 2-3', 12 bonds/uc */
  { 0,  0, 0, { 0, 0, 0}, { 0, 0, 0}, 0}}; /* end-of-table */
#endif /* if 0 = bad old vs. new version */

/* pyrnn = pyr (with same J1=J2) + next nearest bonds, non-checkered 2019-03 */
//    6/2 * nn = 12/uc  J1-bonds  (has m=7/8 magnetization jump, like kago)
//  2*6/2 * nn = 24/uc  J2-bonds
//    6/2 * nn = 12/uc  J3-bonds ?
// double pyrnn_xa[3][3]={{1,0,0},{0,1,0},{0,0,1}}; /* old wrong pyro? geom? */
// -l6 8 0 0  2 1 0  1 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.01019610
// -l6 8 0 0  3 1 0  2 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.01925771 = l3
// -l6 4 0 0  0 2 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.03103972
// -l6 8 0 0  4 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.03103996
// -l6 8 0 0  3 1 0  1 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.03554922 = l3
// -l6 8 0 0  5 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.03603449 = l3
// -l6 8 0 0  2 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.03886395
// -l6 1 -1 0  1 1 0  -2 2 4 nn  8*4 4+28 n1 35960 E0 8.0 8.04029686 u6 0.36405743   0.59161588
// -l6 8 0 0  7 1 0  4 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.04029686 = l3
// -l6 8 0 0  6 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.04507146
// -l6 2 0 0  0 2 0  0 0 2   nn  8*4 4+28 n1 35960 E0 8.0 8.06034778 u6 0.52383046   0.57785279
// -l6 8 0 0  0 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.06388288
// -l6 8 0 0  3 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.07607561 = l3
// -l6 8 0 0  7 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.08101405 = l3
// -l6 8 0 0  1 1 0  0 0 1   nn  8*4 4+28 n1 35960 E0 8.0 8.11489599 = l3
// -l6 4 0 0  0 4 0  0 0 4   nn 64*4 
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J2=-.25 E0/N= 0.35156250 E0=90
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J2=-1/8 E0/N= 0.53515625 E0=137
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J2=-/16 E0/N= 0.62695313 E0=160.5
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J2=-/32 E0/N= 0.67285161 E0=172.25001
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J2=-/64 E0/N= 0.69580508 E0=178.12610016
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J2=0    E0/N= 0.71875    E0=184
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J2=+/64 E0/N= 0.74121840 E0=189.75190988
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J2=.25  E0/N= 1.07821291 E0=276.0225
// J2pyro Nw=9*NN MaxSym=12*NN 2+254 k0 J1=0    E0/N= 1.43138083 E0=366.4335
// J1pyro Nw=3*NN MaxSym=12*NN 2+254 k0 J2=0    E0/N= 0.71875    E0=184
// J1pyro Nw=3*NN MaxSym=12*NN 3+253 k0 J2=0    E0/N= 0.703125   E0=180
// J1pyro Nw=3*NN MaxSym=12*NN 4+252 k0 J2=0    E0/N= 0.6875     E0=176=11*16
// -l6 2 -2 0  2 2 0  0 0 2 nn=16*4 Sym=4*NN k0 4+60 J2=0 E0=32  E0/N=0.5
// -l6 2 -2 0  2 2 0  0 0 2 nn=16*4 Sym=4*NN k0 6+58 J2=0 E0=24  
// -l6 2 -2 0  2 2 0  0 0 2 nn=16*4 Sym=4*NN k0 8+56 J2=0 E0=16 M=7/8-magjump
// -l6 2 -2 0  2 2 0  0 0 2 nn=16*4 Sym=4*NN k0 9+55 J2=0 E0=12.25582120
 double pyrnn_xa[3][3]={{1,1,0},{-1,1,0},{1,0,1}}; /* 2019-03 FCC-UC e1..e3 */
 site_t pyrnn_xs[]={ /* x=right y=away z=up */
/* s#   *e1  *e2  *e3  (in xa[]=UC-base coordinates) */
  { 1, { 0  , 0  , 0  }}, /* site 0 type=1 posXYZ=0,0,0    */
  { 1, { 0  ,-0.5, 0  }}, /* site 1 type=1 posXYZ=1,-1,0 /2 */
  { 1, { 0  , 0  , 0.5}}, /* site 2 type=1 posXYZ=1,0,1 /2 */
  { 1, {-0.5, 0  , 0.5}}, /* site 3 type=1 posXYZ=0,1,1 /2 */
  { 0, { 0  , 0  , 0  }}};  /* end list type=0, syms=... */
 bond_t pyrnn_xb[]={ /* projections: kagome (0,1,1), checkerboard (0,0,1) */
 /* bondtype, site1, site2, UnitCell-shiftXYZ, dXYZ phaseindex=N/A */
 /* type s0 s1  r2[3]      vec[3]_xa-units w  sorted by s1<s2 */ 
#if 1 /* J1: check all sites 3 times + inverse tetraeder */
// gnuplot.splot # 3D-to-2D-projection:
// set view  0.0,0.0  # proj. J1=checker J2=asym_star_inlet 2019-03 buggy?
// set view 56.0,0.0  # proj. J1=kagome J2=asym_kago_inlet  buggyJ2 ?
// set view 90.0,42.0 # proj. J1=kagome (from 1,1,0) 
  { 1,  0, 1, { 0, 0, 0}, { 0  ,-0.5, 0  }, 0}, /* 0-1 tetraeder1 */
  { 1,  0, 2, { 0, 0, 0}, { 0  , 0  , 0.5}, 0}, /* 0-2 phasefactor n.a. */
  { 1,  0, 3, { 0, 0, 0}, {-0.5, 0  , 0.5}, 0}, /* 0-3 */
  { 1,  1, 2, { 0, 0, 0}, { 0  , 0.5, 0.5}, 0}, /* 1-0 + 0-2 = 1-2 */
  { 1,  1, 3, { 0, 0, 0}, {-0.5, 0.5, 0.5}, 0}, /* 1-0+0-2+2-3=1-3 */
  { 1,  2, 3, { 0, 0, 0}, {-0.5, 0  , 0  }, 0}, /* 2-3 */
  { 1,  0, 1, { 0, 1, 0}, { 0  , 0.5, 0  }, 0}, /* 0-1' tetraeder2 */
  { 1,  0, 2, { 0, 0,-1}, { 0  , 0  ,-0.5}, 0}, /* 0-2' */
  { 1,  0, 3, { 1, 0,-1}, { 0.5, 0  ,-0.5}, 0}, /* 0-3'=0-2+2-3 */
  { 1,  1, 2, { 0,-1,-1}, { 0  ,-0.5,-0.5}, 0}, /* 1-2' */
  { 1,  1, 3, { 1,-1,-1}, { 0.5,-0.5,-0.5}, 0}, /* 1-3' */
  { 1,  2, 3, { 1, 0, 0}, { 0.5, 0  , 0  }, 0}, /* 2-3', 12 bonds/uc */
#endif
#if 1  /* ToDo: replace by runtime-bond-bitflag, same sym J1 w/wo J2? */
  /* 2 2nn-neighbours (3th is J3), distance^2=1.5, 4*6=24bonds = (12/2)/site
    try systematic add J1 bond + nonparallel checker-J2(J1') bonds */
/* checked geom. for each bond n-m = 2D-trapez 2019-03-14 marked as "ok" */
/* programming + checking effort for 3D UC was more than 1 hour (ca. 6h) */
  { 2,  0, 2, { 0,-1,-1}, { 0.0,-1.0,-0.5}, 0}, /* 0-1 + 1-2' ok tetra1 */
  { 2,  0, 3, { 1, 0, 0}, { 0.5, 0.0, 0.5}, 0}, /* 0-2 + 2-3' ok  */
  { 2,  0, 1, {-1, 1, 1}, {-1.0, 0.5, 1.0}, 0}, /* 0-3 + 3-1' ok */
  { 2,  1, 3, { 1, 0, 0}, { 0.5, 0.5, 0.5}, 0}, /* 1-2 + 2-3' ok */
  { 2,  1, 2, { 0, 0,-1}, { 0.0, 0.5,-0.5}, 0}, /* 1-0 + 0-2' ok */
  { 2,  2, 3, { 1, 0,-1}, { 0.5, 0.0,-1.0}, 0}, /* 2-0 + 0-3' ok */

  { 2,  0, 3, { 1,-1,-1}, { 0.5,-1.0,-0.5}, 0}, /* 0-1 + 1-3' ok tetra1 */
  { 2,  0, 1, { 0, 1, 1}, { 0.0, 0.5, 1.0}, 0}, /* 0-2 + 2-1' ok */
  { 2,  0, 2, {-1, 0, 0}, {-1.0, 0.0, 0.5}, 0}, /* 0-3 + 3-2' ok */
  { 2,  1, 3, { 1, 0,-1}, { 0.5, 0.5,-0.5}, 0}, /* 1-0 + 0-3' ok */
  { 2,  1, 2, {-1, 0, 0}, {-1.0, 0.5, 0.5}, 0}, /* 1-3 + 3-2' ok */
  { 2,  2, 3, { 1,-1,-1}, { 0.5,-1.0,-1.0}, 0}, /* 2-1 + 1-3' ok */
#if 1
  { 2,  0, 2, { 0, 1, 0}, { 0.0, 1.0, 0.5}, 0}, /* 0-1' + 1-2 ok tetra2=1'  */
  { 2,  0, 3, { 0, 0,-1}, {-0.5, 0.0,-0.5}, 0}, /* 0-2' + 2-3 ok */
  { 2,  0, 1, { 1, 0,-1}, { 1.0,-0.5,-1.0}, 0}, /* 0-3' + 3-1 ok */
  { 2,  1, 3, { 0,-1,-1}, {-0.5,-0.5,-0.5}, 0}, /* 1-2' + 2-3 ok */
  { 2,  1, 2, { 0,-1, 0}, { 0.0,-0.5, 0.5}, 0}, /* 1-0' + 0-2 ok */
  { 2,  2, 3, { 0, 0, 1}, {-0.5, 0.0, 1.0}, 0}, /* 2-0' + 0-3 ok */

  { 2,  0, 3, { 0, 1, 0}, {-0.5, 1.0, 0.5}, 0}, /* 0-1' + 1-3 ok tetra2=1'  */
  { 2,  0, 1, { 0, 0,-1}, { 0.0,-0.5,-1.0}, 0}, /* 0-2' + 2-1 ok */
  { 2,  0, 2, { 1, 0,-1}, { 1.0, 0.0,-0.5}, 0}, /* 0-3' + 3-2 ok */
  { 2,  1, 3, { 0,-1, 0}, {-0.5,-0.5, 0.5}, 0}, /* 1-0' + 0-3 ok */
  { 2,  1, 2, { 1,-1,-1}, { 1.0,-0.5,-0.5}, 0}, /* 1-3' + 3-2 ok */
  { 2,  2, 3, { 0, 1, 1}, {-0.5, 1.0, 1.0}, 0}, /* 2-1' + 1-3 ok */
#endif
  /* J2 : view  0.0,0.0  = square + asym (buggy!) */
  /* J2 : view 56.0,0.0  = partly kagome */
  /* J2 : view 90.0,42.0  = asym */
#endif
#if 0 /* J3: awas also buggy 2019-03-08+11 fixed, 3 J3/site = 3*4 = 12  */
  /* systematic J3 bond as J1 + checker_J2(J1') in same direction, dist^2=2 */
  /* J3 site 0 only: view 57.0,0.0  = trapez-lattice (square) */
  /* J3 site 0 only: view  0.0,0.0  = triangular-lattice */
  /* J3 1st half: view 56.0,0.0  = kagome */
  /* J3 1st half: view  0.0,0.0  = checker-board */
  { 3,  0, 0, { 0,-1, 0}, { 0,-1, 0}, 0}, /* J3 0-1 + 1-0' = - (0-1' + 1-0) */
  { 3,  0, 0, { 0, 0, 1}, { 0, 0, 1}, 0}, /* J3 0-2 + 2-0' */
  { 3,  0, 0, {-1, 0, 1}, {-1, 0, 1}, 0}, /* J3 0-3 + 3-0' */
  { 3,  1, 1, { 0, 1, 1}, { 0, 1, 1}, 0}, /* J3 1-2 + 2-1' */
  { 3,  1, 1, {-1, 1, 1}, {-1, 1, 1}, 0}, /* J3 1-3 + 3-1' */
  { 3,  1, 1, { 0, 1, 0}, { 0, 1, 0}, 0}, /* J3 1-0 + 0-1' */
  { 3,  2, 2, {-1, 0, 0}, {-1, 0, 0}, 0}, /* J3 2-3 + 3-2' */
  { 3,  2, 2, { 0, 0,-1}, { 0, 0,-1}, 0}, /* J3 2-0 + 0-2' */
  { 3,  2, 2, { 0,-1,-1}, { 0,-1,-1}, 0}, /* J3 2-1 + 1-2' */
  { 3,  3, 3, { 1, 0,-1}, { 1, 0,-1}, 0}, /* J3 3-0 + 0-3' */
  { 3,  3, 3, { 1,-1,-1}, { 1,-1,-1}, 0}, /* J3 3-1 + 1-3' */
  { 3,  3, 3, { 1, 0, 0}, { 1, 0, 0}, 0}, /* J3 3-2 + 2-3' */
#endif /* J3 */
  { 0,  0, 0, { 0, 0, 0}, { 0, 0, 0}, 0}}; /* end-of-table */

/* --- m_sc SC = simple cubic 3D (faster than m_sc.c, sym=48*NN) 2016-12 --- */
 double m_sc_xa[3][3]={{1,0,0},{0,1,0},{0,0,1}}; /* base vectors of unitcell */
 site_t m_sc_xs[]={
  { 1, {0.5 ,0.5 ,0.5}}, /* site 0 */
  { 0, {0   ,0   ,0}}};/* end */
 bond_t m_sc_xb[]={
/* type s1 s2 rel_unitcell[3] bond_direction[3] phaseindex */
  { 1,  0, 0, { 1, 0, 0}, { 1 , 0 ,0}, 0}, /*   +x */
  { 1,  0, 0, { 0, 1, 0}, { 0 , 1 ,0}, 1}, /*   +y */
  { 1,  0, 0, { 0, 0, 1}, { 0 , 0 ,1}, 2}, /*   +z */
  { 0,  0, 0, {0,0,0}, {0,0,0}, 0}}; /* end-set */

/* testing FCC via different UC's,  fcc1 via other generators */
// double fcc1_xa[3][3]={{2,0,0},{0,2,0},{0,0,2}}; /* SC = 4/uc nb=12/2n */
// site_t fcc1_xs[]={
//  { 1, {0 ,0 ,0}}, /* site 0 type=1 posXYZ=0,0,0  */
//  { 1, {1 ,1 ,0}}, /* site 1 type=1 posXYZ=1,1,0  */
//  { 1, {1 ,0 ,1}}, /* site 2 type=1 posXYZ=1,0,1  */
//  { 1, {0 ,1 ,1}}, /* site 3 type=1 posXYZ=0,1,1  */
//  { 0, {0 ,0 ,0}}};  /* end list type=0 */
// bond_t fcc1_xb[]={ /* 4*6=24 is to big, skip it */ }
// l7 = fcc2
// compare fcc1  utlf 2 0 0   0 2 0   0 0 2 NN=8*4/(v_uc=8)
// compare fcc2  utlf 1 -1 0  1 1 0  -1 1 2 NN=4*1
// 2D-projection: (0,1,1)=triangular_lattice,  (0,0,1)=J1-J2-square
//    UC                e1      e2       e3
 double fcc2_xa[3][3]={{1,1,0},{-1,1,0},{1,0,1}}; /* FCC-UC 1/uc nb=12/2n */
 site_t fcc2_xs[]={ /* x=right y=away z=up */
/* s#   *e1  *e2  *e3  (in xa[]=UC-base coordinates) */
  { 1, { 0   , 0   , 0}},   /* site 0 type=1 posXYZ=0,0,0    */
  { 0, { 0   , 0   , 0}}};  /* end list type=0, syms=... */
 bond_t fcc2_xb[]={ /* projections: kagome (0,1,1), checkerboard (0,0,1) */
 /* bondtype, site1, site2, UnitCell-shiftXYZ, dXYZ phaseindex=N/A */
 /* type s0 s1  r2[3]      vec[3]_xa-units w  sorted by s1<s2 */ 
  { 1,  0, 0, { 1, 0, 0}, { 1, 0, 0}, 0}, /* ( 1, 1, 0)=e2 tetraeder1 */
  { 1,  0, 0, { 0, 1, 0}, { 0, 1, 0}, 0}, /* (-1, 1, 0)=-e1 phasefactor NA */
  { 1,  0, 0, { 0, 0, 1}, { 0, 0, 1}, 0}, /* ( 1, 0, 1)=e3 */
  { 1,  0, 0, { 0, 1, 1}, { 0, 1, 1}, 0}, /* ( 0, 1, 1)=e2+e3 */
  { 1,  0, 0, {-1, 1, 1}, {-1, 1, 1}, 0}, /* (-1, 0, 1)=e3-e1+e2 */
  { 1,  0, 0, {-1, 0, 1}, {-1, 0, 1}, 0}, /* ( 0,-1, 1)=e3-e1 */
  { 0,  0, 0, { 0, 0, 0}, { 0, 0, 0}, 0}};


#define Ns 100  /* maximum number of sites (per unitcell) */
#define Nb 800  /* maximum number of bonds (per unitcell) */

int dim=2, nn=0;
const char *title="no title";

// Apr09: we always define unitcell with unique vectors?
int    xe[3][3]={{1,0,0},{0,1,0},{0,0,1}}; /* vol=1 */
// int xe[3][3]={{0,1,1},{1,0,1},{1,1,0}}; /* vol=2 FCC */
// int xe[3][3]={{2,1,1},{1,2,1},{1,1,2}}; /* vol=4 */
/* unitcell specifique: input */
double (*xa)[3] = ska_xa; /* = (double (*)[3])til11_xa; */
site_t *xs      = ska_xs;
bond_t *xb      = ska_xb;
/* full cluster (bad: global work arrays) = output */
site_t *ys      = NULL;
bond_t *yb      = NULL;

static struct {
  const char *xn;  /* name (const) */
  double (*xa)[3]; /* base system */
  site_t *xs;      /* site definition */
  bond_t *xb;      /* bond definition */
} uclist[]={  // ToDo: optional from file
  {"tiling11",                     til11_xa,til11_xs,til11_xb}, // l0
  {"kagome (tiling8)",              til8_xa, til8_xs, til8_xb}, // l1
  {"stretched kagome",              ska_xa,  ska_xs,  ska_xb }, // l2
  {"pyrochlore (3D checker board)", pyr_xa,  pyr_xs,  pyr_xb }, // l3
  {"triangle (tiling1)",            til1_xa, til1_xs, til1_xb}, // l4
  {"simple cubic (SC)",             m_sc_xa, m_sc_xs, m_sc_xb}, // l5
  {"pyrochlore J1-J2(-J3)",      pyrnn_xa, pyrnn_xs, pyrnn_xb}, // l6
  {"face centered cubic (FCC)",   fcc2_xa,  fcc2_xs,  fcc2_xb}, // l7
  {NULL, NULL, NULL, NULL}}; /* end of list */

/* see edgevec.c */
/* 2D-rotations det(T)=+1 oder als Permutation von sites tabellieren? */
int rotate[2][3][3]={
  {{1, 0, 0}, { 0, 1, 0}, { 0, 0, 1}},
  {{0, 1, 0}, {-1, 0, 0}, { 0, 0, 1}},
};         
/* 2D-reflections det(T)=+1 oder als Vectoren \pm 1 */
int reflect[2][3][3]={
 {{ 1, 0, 0}, { 0, 1, 0}, { 0, 0, 1}},
 {{-1, 0, 0}, { 0,-1, 0}, { 0, 0, 1}},
};         


int    get_deti(int    e[3][3]){   /* volume=det(a) */
 return (
     e[0][0]*(e[1][1]*e[2][2]-e[1][2]*e[2][1])   /* 00(11 22 - 12 21) */ 
    +e[0][1]*(e[1][2]*e[2][0]-e[1][0]*e[2][2])   /* 01(12 20 - 10 22) */
    +e[0][2]*(e[1][0]*e[2][1]-e[1][1]*e[2][0])); /* 02(10 21 - 11 20) */
}

double get_detd(double e[3][3]){   /* volume=det(a) */
 return (
     e[0][0]*(e[1][1]*e[2][2]-e[1][2]*e[2][1])   /* 00(11 22 - 12 21) */ 
    +e[0][1]*(e[1][2]*e[2][0]-e[1][0]*e[2][2])   /* 01(12 20 - 10 22) */
    +e[0][2]*(e[1][0]*e[2][1]-e[1][1]*e[2][0])); /* 02(10 21 - 11 20) */
}

int sprod(int a[3], int b[3]){  /* spat-product */
   return a[0]*b[0]+a[1]*b[1]+a[2]*b[2]; }

/* y = x1 + a * x2  */
double *add( double x1[3], double x2[3], double a, double y[3]){
   int i;
   for (i=0;i<3;i++) y[i] = x1[i] + a*x2[i];
   return y;
}

/* y = matrix_a * x  */
double *mltply( double a[3][3], double x[3], double y[3]){
   int i;
   if (x==y) { fprintf(stderr,"&y==&x"); exit(1); }
   for (i=0;i<3;i++) 
     y[i] = a[0][i]*x[0] + a[1][i]*x[1] + a[2][i]*x[2];
   return y;
}

/* better return *[3][3] (*i_matrix_t) */
int product(int a[3][3],int u[3][3],int au[3][3]){ /* au=a*u */
   int i,j,b[3][3]={{0,0,0},{0,0,0},{0,0,0}}; /* buffer if &au=&a */
   for (i=0;i<3;i++) for (j=0;j<3;j++)
     b[i][j]  = a[i][0]*u[0][j] + a[i][1]*u[1][j] + a[i][2]*u[2][j];
   for (i=0;i<3;i++) for (j=0;j<3;j++)
     au[i][j] = b[i][j];
   return 0;
}

int dotprod(int a[3][3],int b[3][3],int ab[3][3]){ /* ab=a.b */
   int i,j;
   for (i=0;i<3;i++)
   for (j=0;j<3;j++) ab[i][j] = sprod(a[i],b[j]);
   return 0;
}

/* TransponiertAdjuncte=det(A)*A^(-1); -> Bildung der Inversen Matrix */
int adjunct(int a[3][3],int adjtrans[3][3]){
   if(adjtrans==a){ fprintf(stderr,"---error1---\n");exit(1); }
   adjtrans[0][0]=a[1][1]*a[2][2]-a[1][2]*a[2][1];
   adjtrans[0][1]=a[2][1]*a[0][2]-a[2][2]*a[0][1];
   adjtrans[0][2]=a[0][1]*a[1][2]-a[0][2]*a[1][1];
   adjtrans[1][0]=a[1][2]*a[2][0]-a[1][0]*a[2][2];
   adjtrans[1][1]=a[2][2]*a[0][0]-a[2][0]*a[0][2];
   adjtrans[1][2]=a[0][2]*a[1][0]-a[0][0]*a[1][2];
   adjtrans[2][0]=a[1][0]*a[2][1]-a[1][1]*a[2][0];
   adjtrans[2][1]=a[2][0]*a[0][1]-a[2][1]*a[0][0];
   adjtrans[2][2]=a[0][0]*a[1][1]-a[0][1]*a[1][0];
   return 0;
}

int compare(int a1[3][3], int a2[3][3]){ /* 2x2 */
    int bb1[3][3],bb2[3][3],i1,i2,v;
    /* compare  A1*T_i*A2^-1=int-matrix? */
    /* rechts - links Reihenfolge beachten! */
    v=abs(get_deti(a2));
    for(i1=0;i1<2*2;i1++){	/* all rotations+reflections */
       adjunct(a2,bb1);
       product( rotate[i1/2],bb1,bb2);
       product(reflect[i1%2],bb2,bb1);
       product(  a1,         bb1,bb2);
       for(i2=0;i2<2*2;i2++)if(abs(bb2[i2/2][i2%2])%(v)!=0)break;
       if(i2>=2*2){
         fprintf(stderr,"# V=%2u ROT=%d REF=%d U=(%d %d, %d %d)\n",
           v,i1/2,i1%2,
           bb2[0][0]/v,bb2[0][1]/v,
           bb2[1][0]/v,bb2[1][1]/v);
         break;
       }   
    }      
    if(i1==2*2) { /* fprintf(stderr,"# a1!=a2\n"); */ return -1;}
    return 0;
}

int minmax(int e[3][3],int emin[3],int emax[3]){
   /* einhuellenden Kubus ermitteln (alle eck-koords checken) */
   int i,j,k;
   for(i=0;i<3;i++){                     
     emin[i]=emax[i]=e[0][i]+e[1][i];    
     if(emin[i]>0)emin[i]=0;
     if(emax[i]<0)emax[i]=0;
     for(j=0;j<3;j++){
       if(e[j][i]<emin[i])emin[i]=e[j][i];
       if(e[j][i]>emax[i])emax[i]=e[j][i];
       for(k=j+1;k<3;k++){
         if(e[j][i]+e[k][i]<emin[i])emin[i]=e[j][i]+e[k][i];
         if(e[j][i]+e[k][i]>emax[i])emax[i]=e[j][i]+e[k][i];
   }}}
#if 0                                   
   printf("# emin=(%d %d %d) emax=(%d %d %d)\n", 
          emin[0],emin[1],emin[2],emax[0],emax[1],emax[2]);
#endif
   return 0;
}

/* --- from m_sc.c for simple cubic lattice --- */
#define  NN    64000	/* max. number of vertices (unitcells) 40^3=64e3 */
int vertex[NN][3];	/* unitcell-position = (x,y,z) (same as SC) */
#define F0(txt)	{ fprintf(stderr,"\nERROR L%d: " txt "\n",__LINE__);exit(1); }

/* ================== 3D-lattices ===========================  
   -> utils/cubes.c 
   Berechnung kubischer Gitter aus Vektoren die Elementarzelle aufspannen 

   unit cells automatisch erzeugen durchprobieren mit l_max=N, e1<e2<e3

+++TeX+++ ---------------------------------------------------------------

 Beispiel in zwei Dimensionen. $a=(a_x,a_y)$ und $b=(b_x,b_y)$ sind die 
 zwei Vektoren, die die Elementarzelle aufspannen. 
 Sie sind im allgemeinen nicht senkrecht zueinander. 
 Der Vektor $r=(r_x,r_y)$ liegt innerhalb der Elementarzelle, wenn seine
 Komponenten $(r_a,r_b)$ bezueglich der Elementarvektoren zwischen $0$ und $1$
 liegen.
 \begin{eqnarray*}
      r &=& r_a a   + r_b b \\
 %
    a r &=& r_a a^2 + r_b a b \\
    b r &=& r_a a b + r_b b^2 \\
 %
    r_a &=& \frac{(b^2a-(a,b)b)r}{a^2b^2-(a,b)^2} \\
    r_b &=& \frac{(a^2b-(a,b)a)r}{a^2b^2-(a,b)^2} \\
 \end{eqnarray*}
 Grenzfall $(a,b)=0$ liefert bekannte Beziehungen. 
 Im 3 dimensionalen muss entsprechend ein Gleichungssystem 3.\ Ordnung 
 geloest werden.
 \begin{verbatim}
  maple> solve( { y0=ra*aa+rb*ab+rc*ac,\ 
  maple>          y1=ra*ab+rb*bb+rc*bc,\
  maple>          y2=ra*ac+rb*bc+rc*cc},{ra,rb,rc});
  {ra= ( + (bb cc - bc bc) y0 + (ac bc - cc ab) y1 + (bc ab - ac bb) y2)/%1,
   rb=-( + (ab cc - bc ac) y0 + (ac ac - cc aa) y1 + (bc aa - ac ab) y2)/%1,
   rc= ( + (bc ab - bb ac) y0 + (ab ac - aa bc) y1 + (aa bb - ab ab) y2)/%1}
  %1 := + ac bc ab + ac bc ab + bb cc aa - bc bc aa - ac ac bb - ab ab cc
  Matrixform: x=A*r; r=Adj(A)*r/Det(A)
  \end{verbatim}
  Auf die Division durch die Determinante kann evl. verzichtet werden, in dem
  die Einheit auf Det(a,b,c) gesetzt wird.
  Hilfreich koennte die Speicherung der Determinante der Einheitszelle und
  der inversen Elementarmzelle (normiert mit der Determinante) sein.
   ---TeX--- ----------------------------------------------------- */

/* vertex -> (x,y,z) */
int get_koord(const int i,int s[3]){
  if(i<0 || i>=nn) { fprintf(stderr,"error: get_koord\n"); return -1; }
  s[0]=vertex[i][0];
  s[1]=vertex[i][1];
  s[2]=vertex[i][2];
  return 0;}
  
/* (x,y,z) -> vertex */  
int get_vertex(int xx[3]){
  int i,x[3],(*e)[3]=xe,aa[3][3],bb[3][3],ry[3],ra[3],det,(*s)[3]=vertex;
  /* ra = adjunct(aa)/det(aa) * ry = aa^(-1) * ry */
  dotprod(e,e,aa); /* aa=e.e */
  det=get_deti(aa); adjunct(aa,bb);
  for(i=0;i<3;i++) ry[i]=sprod(e[i],xx);
  /* ra = adjunct(aa) * ry */
  for(i=0;i<3;i++) ra[i]= bb[i][0]*ry[0] + bb[i][1]*ry[1] + bb[i][2]*ry[2];
  for(i=0;i<3;i++) if(ra[i]<0)ra[i]=-((-ra[i]-1)/det)-1;else ra[i]/=det;
  for(i=0;i<3;i++) x[i]=xx[i]-ra[0]*e[0][i]+ra[1]*e[1][i]+ra[2]*e[2][i];
  for(i=0;i<nn;i++) if(x[0]==s[i][0] && x[1]==s[i][1] && x[2]==s[i][2]) return i;
  fprintf(stderr,"error get_vertex: %2d %2d %2d (%2d %2d %2d) => %2d %2d %2d not found\n",
    xx[0],xx[1],xx[2],ra[0],ra[1],ra[2],x[0],x[1],x[2]);
  return -1;}

int q2(int x){return x*x;}

int vertex_init(){ /* vertex_init */
   int ii,i,r[3],ra[3],aa[3][3],bb[3][3],ry[3],det,emin[3],emax[3],
     (*s)[3]=vertex, (*e)[3]=xe /* ,*a=e[0],*b=e[1],*c=e[2] */ ;
   /* plaetze finden! */
   
   /* einhuellenden Kubus ermitteln (alle eck-koords checken) */
   minmax(e,emin,emax);
   
   ii=0;
   /* ra = adjunct(aa)/det(aa) * ry = aa^(-1) * ry */
   dotprod(e,e,aa); /* aa=e.e */
   det=get_deti(aa); adjunct(aa,bb);
   for(r[0]=emin[0];r[0]<=emax[0];r[0]++)
   for(r[1]=emin[1];r[1]<=emax[1];r[1]++)  /* ineffektiv! */
   for(r[2]=emin[2];r[2]<=emax[2];r[2]++){
    for(i=0;i<3;i++) ry[i]=sprod(e[i],r);
    /* ra = adjunct(aa) * ry */
    for(i=0;i<3;i++) ra[i]= bb[i][0]*ry[0] + bb[i][1]*ry[1] + bb[i][2]*ry[2];
    if(ra[0]>=0 && ra[1]>=0 && ra[2]>=0 && ra[0]<det && ra[1]<det && ra[2]<det){
      s[ii][0]=r[0];
      s[ii][1]=r[1];
      s[ii][2]=r[2]; ii++; 
      if (ii>=NN) {printf("ERR: NN to small\n");exit(1);}
    }
    else /* printf("\n") */ ;
   }
   if(ii!=nn)fprintf(stderr,"error vertex_init i!=N i=%d N=%d\n",ii,nn);
   /* --- now s=vertex is initialized --- */
   return 0;
}


/*  --------------------------
 test if R lying in a parallel epiped, move r to x (by n*e)
 */
int inside(int r[3],int e[3][3],int x[3]){  /* 2x2 */
   int i,aa[3][3],bb[3][3],det,ra[3],ry[3],ret=0;
   /* ra = adjunct(aa)/det(aa) * ry = aa^(-1) * ry */
   dotprod(e,e,aa); /* aa=e.e */
   det=get_deti(aa); adjunct(aa,bb);
   for(i=0;i<3;i++) ry[i]=sprod(e[i],r);
   /* ra = adjunct(aa) * ry */
   for(i=0;i<3;i++) ra[i]= bb[i][0]*ry[0] + bb[i][1]*ry[1] + bb[i][2]*ry[2];
   if(ra[0]>=0 && ra[1]>=0 && ra[2]>=0 && ra[0]<det && ra[1]<det && ra[2]<det) ret=1;
   /* ret=1: ok, its lying inside */
   /* else R is outside of E */
   if(x){
     for(i=0;i<3;i++) x[i]=r[i];
     for(i=0;i<3;i++) if(ra[i]<0)ra[i]=-((-ra[i]-1)/det)-1;else ra[i]/=det;
     for(i=0;i<3;i++) x[i]-=ra[0]*e[0][i]+ra[1]*e[1][i]+ra[2]*e[2][i];           
   }
#if 0
   printf("# inside  r(%p)= % d % d % d rc=%d\n",r,r[0],r[1],r[2],ret);
   if(x) printf("# inside rr(%p)= % d % d % d\n",      x,x[0],x[1],x[2]);
#endif
   return ret;
}

/* ---------------------------------------------------------------
   xa=base_vectors unit_cell
   xe=edge_vectors
   xs=sites_of_unitcell
   xb=bonds_of_unitcell  { type, site1, site2, r2[3], ... }
   ns=sites_per_unitcell
   nb=bonds_per_unitcell
   ex: ( 2, 1) v=5    idx <=> (x,y,z) in xa-units via xe -> convert-fkt
       (-1, 2)          0              0 1 2 3 0
       0                2 0  (2,1)     4 5 6 7 4
   0 1 2              0 1    (1,2)     0 1 2 3 0  (4,0)
     3 4 0              0                         (0,2)
     0
   numbering is not best (see upper picture for a better variant)
   --------------------------------------------------------------- */

/* input: xa = base, xe = edgevectors, xs, xb = unitcell, 
 * output:  ys, yb = whole lattice   y=f(x,xe)  */
int ini_y(double xa[3][3], int xe[3][3], site_t *xs, bond_t *xb){
    int i, j, k, n, m, ns, nb, ve, r[3], rr[3];
    double va;

    ve=get_deti(xe); /* topological volume edgevectors */
    va=get_detd(xa); /* geometrical volume unit cell */
    for(ns=0;ns<Ns;ns++) if (!xs[ns].type) break; /* num of sites */
    for(nb=0;nb<Nb;nb++) if (!xb[nb].type) break; /* num of bonds */
    if (ns==Ns) { printf("ERR: Ns to small\n"); exit(1); }
    if (nb==Nb) { printf("ERR: Nb to small\n"); exit(1); }
    fprintf(stderr, "DGB: uc.ns,nb= %d %d  vuc=%.3f  ve=%d\n",ns,nb,va,ve);
    ys=(site_t *)malloc(sizeof(site_t)*(ns*ve+1));
    yb=(bond_t *)malloc(sizeof(bond_t)*(nb*ve+1));
    if((!ys) || (!yb)) { fprintf(stderr,"malloc failed\n"); exit(1); }
    for(i=0; i<ve; i++){ /* all unitcells */
     get_koord(i,r);
     for (n=0; n<ns; n++) {
       j=n+ns*i;
       ys[j].type   = xs[n].type;
//     ys[j].pos[0] = r[0]*xa[0][0] + r[1]*xa[1][0] + r[2]*xa[2][0] + xs[n].pos[0];
//     ys[j].pos[1] = r[0]*xa[0][1] + r[1]*xa[1][1] + r[2]*xa[2][1] + xs[n].pos[1];
//     ys[j].pos[2] = r[0]*xa[0][2] + r[1]*xa[1][2] + r[2]*xa[2][2] + xs[n].pos[2];
       ys[j].pos[0] = r[0] + xs[n].pos[0];
       ys[j].pos[1] = r[1] + xs[n].pos[1];
       ys[j].pos[2] = r[2] + xs[n].pos[2];
     }
    }
    ys[ns*ve].type=0;

    for(i=0; i<ve; i++){ /* all unitcells */
     get_koord(i,r);
     for (n=0; n<nb; n++) {
      rr[0]=r[0]+xb[n].r2[0];
      rr[1]=r[1]+xb[n].r2[1];
      rr[2]=r[2]+xb[n].r2[2];   /* neighbourcell */
      j=i; k=1;
      if( xb[n].r2[0] || xb[n].r2[1] || xb[n].r2[2] ){  /* s2 in other cell */
        /* move rr inside of e */
        k=inside(rr,xe,rr); /* k=0: outside */
        j=get_vertex(rr);
      }
      m=n+nb*i;
      yb[m].type   = xb[n].type;
      yb[m].s1     = xb[n].s1+ns*i;
      yb[m].s2     = xb[n].s2+ns*j;
      yb[m].vec[0] = xb[n].vec[0];
      yb[m].vec[1] = xb[n].vec[1];
      yb[m].vec[2] = xb[n].vec[2];
      yb[m].r2[0]  = ((k)?0:xb[n].r2[0]); /* not correct! */
      yb[m].r2[1]  = ((k)?0:xb[n].r2[1]);
      yb[m].r2[2]  = ((k)?0:xb[n].r2[2]);
      yb[m].w      = xb[n].w;
     }
    } /* for(i=0; i<ve; i++) ... all unitcells */
    return 0;
} /* ini_y */

/* xs,xb is expanded to ys and yb (here xs, xb) on caller!  output daten.def
 *          lattice   base_vecs       edge_vecs     sites   ys  bonds   yb */
int out_def(int lat, double xa[3][3], int xe[3][3], site_t *xs, bond_t *xb){
    int ve, i, k, s1, s2, ns, nb;
    double rr[3];

    ve=get_deti(xe);

    printf("# def-file, automaticly generated by lattice.c %s JS\n"
           "#   l=%d %s dim=%d e= %d %d %d  %d %d %d  %d %d %d  ve=%d\n",
           versionstring, lat, title, dim,
           xe[0][0],xe[0][1],xe[0][2],
           xe[1][0],xe[1][1],xe[1][2],
           xe[2][0],xe[2][1],xe[2][2],ve);

    /* until Apr09 base was always 1 0 0 1, which is bad
     * for non-orthogonal coordinate systems (crooked numbers etc.)
     */ 
    printf("#===base===");
    for (i=0;i<dim*dim;i++){
      if (i%dim==0) printf(" "); /* add gap for the eyes */
      printf(" %.4f", xa[i/dim][i%dim]);
    }
    printf("\n# everything below is now in base coordinates\n");

    /* not sure if I originally would show uc in base coordinates or not */
    /* ToDo: is this the base uc or the frame of PBC? think the base uc */
    /* at moment always E ? */
    printf("#===uvec==="); /* t==e */
    for (i=0;i<dim*dim;i++){
      if (i%dim==0) printf(" "); /* add gap for the eyes */
      printf(" %.4f", ((i/dim==i%dim)?1.0:0.0));
    }
    printf("\n");

    for(i=0;i<3;i++)
      printf("# %c=(%2d %2d %2d)\n",'a'+i,xe[i][0],xe[i][1],xe[i][2]);
    printf("# geometrical volume of unitcell % .6f\n",get_detd(xa));
    // better: out(prod(a,b,buf),dim)
    // Apr09: old version was output= xe*xa, now its xe only
    // output edge frame for PBC
    printf("#===edge==="); /* t==e */
    for (i=0;i<dim*dim;i++) { 
      if (i%dim==0) printf(" "); /* add gap for the eyes */
      printf(" %2d", xe[i/dim][i%dim]);
    }
    printf("\n");

    /* frame for 2dim PBC (0,0)-(0,1)-(1,1)-(1,0)-close */
    if (dim==2)
    printf("#===pbcf=== 0 0  %d %d  %d %d  %d %d\n",
            xe[0][0],          xe[0][1],
            xe[0][0]+xe[1][0], xe[0][1]+xe[1][1], 
            xe[1][0],          xe[1][1]);
            
    printf("#+++model+++\n");
    // ToDo: better give num vertices via arg
    for(ns=0;ns<ve*Ns;ns++) if (!xs[ns].type) break; /* num of sites */
    for(nb=0;nb<ve*Nb;nb++) if (!xb[nb].type) break; /* num of bonds */
    printf("# NN=Number_of_sites Nw=Number_of_bonds Nv=Number_of_bonds(2)\n");
    printf("%d %d 0\n",ns,nb);
    printf("# Sites (s=siteindex  rXYZ=position  X#=parameter_index fz=field)\n");
    printf("# s  rX  rY  rZ [e#] [U#] [fz#]\n");
    for(i=0; i<ns; i++){
      // mltply(xa,xs[i].pos,rr);  // rr = xa * xs
      // printf(" %2d % 5.3f % 5.3f % 5.3f e8 u7\n",i, rr[0], rr[1], rr[2]);
      // Apr09: now in base coords (xa)
      printf(" %2d % 5.3f % 5.3f % 5.3f e8 u7\n",
        i, xs[i].pos[0], xs[i].pos[1], xs[i].pos[2]);
    }

    /* p_w is the index to angle for twisted boundary conditions */
    /* ToDo: printf("# Bond s1 s2 [R= ...] [t#] [J#] [V#] [w#]\n"); */
    printf("# Bond s1 s2 [R= ...] [t#] [J#] [V#] [w#]\n");
    for(i=0; i<nb; i++){ /* all unitcells */
      k=0;
      if( xb[i].r2[0] || xb[i].r2[1] || xb[i].r2[2] ) k=1;
      s1=xb[i].s1;
      s2=xb[i].s2;
      /* r1=base_A*vec; */
      /* unsure: mltply(xa,xb[i].vec,r1); */ /* bond_s1 (**basevecs???) */
      add(xb[i].vec,rr,0,rr);
      /* TODO Richtungsvektor bond_t.vec fuer PBC angeben! 0..+9 = 0,n,s,e,w,ne,nw,se,sw */
      /* TODO Aug09: for bond to outside UC "+t0 ..." was a bad (old) syntax */
      /* R= helpful for plotting 2D/3D and magnetic-field factors */
      printf(" %2d %2d %2d  R= %5.2f %5.2f %5.2f  t0 j%d v0 w%d %c\n",
        i,s1,s2, /* R= */ rr[0], rr[1], rr[2],
        xb[i].type,xb[i].w, ((k)?'+':' '));
    }
    printf("#---model---\n");
    return 0;
} /* out_def */

/* minimalistic output for gnuplot 3D-plot, 2019-03-05
 *            base_vecs       edge_vecs     sites.exp    bonds.expanded  */
int out_splot(int lat, double xa[3][3], int xe[3][3], site_t *xs, bond_t *xb){
    int i,j,ns,nb,s1,s2,ve;
    double zoom=10.0, rr[3], r1[3], r2[3], r3[3], r4[3];
    double va;
    const char *farb="C"; /* declaration for color */
    int maxcolor=1;

    ve=get_deti(xe); /* must be positive defined */ 
    va=get_detd(xa); /* should be +1.0000 */
    printf("# gnuplot output - gnuplot version 4.6 (2013)\n");
    printf("# lattice.c  l%d  title= %s\n", lat, title);
    printf("# base_vol_UC= %.3f   edge_vol= %d\n", va, ve);
    printf("# set view rot_x,rot_z,scale,scale_z # screen=(x,y)\n");
    printf("# set view  0,0  # from z to xy l6=checker l7=J1-J2-square\n");
    printf("# set view 56,0  # from z+y     l6=kago    l7=triangular\n");
    printf("set xlabel \"x\"\n");
    printf("set ylabel \"y\"\n");
    printf("set zlabel \"z\"\n");
    /* UC-output added 2019-03-10 UC.e1..3
     *   shifted by (0,0,1)*0.05 to move outside of potential bonds  */
    printf("# UC e1 e2 e3 shifted by (0,0,1)*0.05*zoom\n"
           "set label 9001 \"O\" at %8.4f,%8.4f,%8.4f # origin 0,0,0\n"
           "set label 9002 \"e1\" at %8.4f,%8.4f,%8.4f  # uc.e1\n"
           "set label 9003 \"e2\" at %8.4f,%8.4f,%8.4f  # uc.e2\n"
           "set label 9004 \"e3\" at %8.4f,%8.4f,%8.4f  # uc.e3\n",
           0.0, 0.0, 0.05*zoom,
           xa[0][0]*zoom, xa[0][1]*zoom, (xa[0][2]+0.05)*zoom,  /* uc.e1 */
           xa[1][0]*zoom, xa[1][1]*zoom, (xa[1][2]+0.05)*zoom,  /* uc.e2 */
           xa[2][0]*zoom, xa[2][1]*zoom, (xa[2][2]+0.05)*zoom); /* uc.e3 */
    for(ns=0;ns<ve*Ns;ns++) if (!xs[ns].type) break; /* num of sites */
    for(nb=0;nb<ve*Nb;nb++) { /* bond loop */
      if (maxcolor < xb[nb].type) maxcolor = xb[nb].type;
      if (!xb[nb].type) break; /* num of bonds */
    }
    for(i=0; i<ns; i++){ /* all unitcells */
      mltply(xa,xs[i].pos,rr); /* rr = matrix_xa * xs = xb[].vec (?) */
      /* ToDo: plot only 2 first unitcells */
      if (i>9) printf("#"); /* outcomment */
      printf("set label %3d \"%d\" at %7.3f,%7.3f,%7.3f # shift by 1,1,1\n", 
        i+1, i, zoom*(rr[0]+0.1), zoom*(rr[1]+0.1), zoom*(rr[2]+0.1));
    }
    
    printf("splot \"-\" u 1:2:3 t \"UC\" w l lw 1 lc 7,\\\n");
    for (j=0;j<=maxcolor;j++){ /* colors in blocks of same color */
      printf(" \"-\" u 1:2:3 t \"J%d\" with lines lw %d lc %d",
        j+1,((j)?1:3),j+1);
      if (j<=maxcolor-1) printf(",\\");
      printf("\n");
    }
    /* UC-output added 2019-03-10 2D-UC + UC.e3 ...
     *   shifted by (0,0,1)*0.05 to move outside of potential bonds  */
    printf("# UC e1 e2 e3 shifted by (0,0,1)*0.05*zoom\n"
           "% 8.4f % 8.4f % 8.4f  # origin 0,0,0\n"
           "% 8.4f % 8.4f % 8.4f  # uc.e1\n"
           "% 8.4f % 8.4f % 8.4f  # uc.e1+e2\n"
           "% 8.4f % 8.4f % 8.4f  # uc.e2\n"
           "% 8.4f % 8.4f % 8.4f  # origin 0,0,0\n"
           "% 8.4f % 8.4f % 8.4f  # uc.e3\ne\n",
           0.0, 0.0, 0.05*zoom,
           xa[0][0]*zoom, xa[0][1]*zoom, (xa[0][2]+0.05)*zoom,  /* uc.e1 */
           (xa[0][0]+xa[1][0])*zoom,
            (xa[0][1]+xa[1][1])*zoom,
             (xa[0][2]+xa[1][2]+0.05)*zoom,  /* uc.e1+e2 */
           xa[1][0]*zoom, xa[1][1]*zoom, (xa[1][2]+0.05)*zoom,  /* uc.e2 */
           0.0, 0.0, 0.05*zoom,
           xa[2][0]*zoom, xa[2][1]*zoom, (xa[2][2]+0.05)*zoom); /* uc.e3 */
    for (j=0;j<=maxcolor;j++){ /* colors in blocks of same color */
      for(i=0; i<nb; i++){ /* bond-loop, all unitcells (ToDo?) */
        if (j+1!=xb[i].type) continue; /* skip other colors here */
        s1=xb[i].s1; /* site 1 already shifted to pos */
        s2=xb[i].s2; /* site 2 already shifted to pos */
        /* 0.5 forward + -0.5 backward  or 1.0 forward + 0.0 backward */
        /* rr=xs+0*rr;                     r1=base_A*rr; */
        add(xs[s1].pos,rr,0,rr);           mltply(xa,rr,r1); /* 1st half */
        add(xs[s1].pos,xb[i].vec, 0.5,rr); mltply(xa,rr,r2);
        add(xs[s2].pos,rr,0,rr);           mltply(xa,rr,r3); /* 2nd half */
        add(xs[s2].pos,xb[i].vec,-0.5,rr); mltply(xa,rr,r4);
        farb="C";
        // printf("# bond %2d ,  %2d %2d , %2d\n",i,s1,s2,xb[i].type);
  #ifdef Layer1m11   /* print only [1,-1,1] layer (rgbt t=transparency) */
        if( (r1[0]-r1[1]+r1[2])*(r1[0]-r1[1]+r1[2])>0.8
         || (r3[0]-r3[1]+r3[2])*(r3[0]-r3[1]+r3[2])>0.8) farb="T";
  #endif
        printf("% 7.3f % 7.3f % 7.3f\n% 7.3f % 7.3f % 7.3f "
               "# b%d:  s%d-s%d J%d col %s%d\n\n\n",
                  zoom*r1[0], zoom*r1[1], zoom*r1[2],
                  zoom*r2[0], zoom*r2[1], zoom*r2[2],
                    i, s1, s2, xb[i].type, farb, j%4 );
        printf("% 7.3f % 7.3f % 7.3f\n% 7.3f % 7.3f % 7.3f "
               "# invers-direction\n\n\n",
                  zoom*r3[0], zoom*r3[1], zoom*r3[2],
                  zoom*r4[0], zoom*r4[1], zoom*r4[2]);
      } /* loop bond index i */
      printf("e\n# next colored type bond follows ...\n");
    } /* loop colors j */
    printf("pause -1\n# END of model\n");
    return 0;
} /* out_splot */


/* besser nur halblange bonds in alle Richtungen zeichnen? */
int out_pov(int lat, double xa[3][3], int xe[3][3], site_t *xs, bond_t *xb){
    int i,j,ns,nb,s1,s2, v;
    double zoom=10.0, rr[3], r1[3], r2[3], r3[3], r4[3];
    const char *farb="C"; /* declaration for color */

    v=get_deti(xe);
    printf("// povray - raytracer version 2.0\n");
    printf("// lattice.c l%d  title= %s\n", lat, title);
    printf(
"// clock=0..1 for stereoview\n"
"#include \"colors.inc\"\n"
"#include \"shapes.inc\"\n"
"#include \"chars.inc\"\n"
"camera {\n"
"   location  <10+clock*5, 10,-60> // right,up,away\n"
"   look_at   <20,  20,  10>\n"
"}\n"
"light_source {< 20, 80, -160> colour White}\n"
"background { color White }\n"
"union {  // Drei-Bein\n"
" object { char_X scale 0.4 translate <11,0,0> }\n"
" object { char_Y scale 0.4 translate <0,11,0> }\n"
" object { char_Z scale 0.4 translate <0,0,11> }\n"
" cone { < 0, 0, 0>,0.2  < 8,0,0>,0.2 }\n"
" cone { < 0, 0, 0>,0.2  <0, 8,0>,0.2 }\n"
" cone { < 0, 0, 0>,0.2  <0,0, 8>,0.2 }\n"
" cone { < 8, 0, 0>,0.8  <10,0,0>,0.0 }\n"
" cone { < 0, 8, 0>,0.8  <0,10,0>,0.0 }\n"
" cone { < 0, 0, 8>,0.8  <0,0,10>,0.0 }\n"
" pigment { color rgb < 0.7,0.7,0.7 > }\n"
" no_shadow\n"
" translate <-10, -5,-10>\n"
"}\n"
"#declare Site = \n"
" sphere { <0, 0, 0>, 1\n"
"   pigment { color rgb <0.4,0.4,0.8> }\n"
"   finish { ambient 0.2 diffuse 0.9 // refraction 1\n"
"   }\n"
"   no_shadow\n"
" }\n"
"#declare C0 = rgb < 0.95,0.95,0.25 >;\n"
"#declare C1 = rgb < 0.25,0.95,0.25 >;\n"
"#declare C2 = rgb < 0.95,0.25,0.25 >;\n"
"#declare C3 = rgb < 0.95,0.25,0.95 >;\n"
"#declare T0 = rgbt < 0.95,0.95,0.25,0.8 >;\n"
"#declare T1 = rgbt < 0.25,0.95,0.25,0.8 >;\n"
"#declare T2 = rgbt < 0.95,0.25,0.25,0.8 >;\n"
"#declare T3 = rgbt < 0.95,0.25,0.95,0.8 >;\n"
);

    for(ns=0;ns<v*Ns;ns++) if (!xs[ns].type) break; /* num of sites */
    for(nb=0;nb<v*Nb;nb++) if (!xb[nb].type) break; /* num of bonds */
    for(i=0; i<ns; i++){ /* all unitcells */
      mltply(xa,xs[i].pos,rr);
      printf("object { Site scale 1.0 translate <% 6.3f,% 6.3f,% 6.3f> } // s%d\n",
         zoom*rr[0], zoom*rr[1], zoom*rr[2],i);
    }

    for(i=0; i<nb; i++){ /* all bonds (of multiplied unitcells) */
      j=xb[i].type;
      s1=xb[i].s1;  /* first site */
      s2=xb[i].s2;  /* neigbour site */
      add(xs[s1].pos,rr,0,rr);           mltply(xa,rr,r1); /* rr=xs+0*rr; r1=A*rr; */
      add(xs[s1].pos,xb[i].vec, 0.5,rr); mltply(xa,rr,r2);
      add(xs[s2].pos,rr,0,rr);           mltply(xa,rr,r3);
      add(xs[s2].pos,xb[i].vec,-0.5,rr); mltply(xa,rr,r4);
      farb="C";
      printf("// bond %2d ,  %2d %2d , %2d\n",i,s1,s2,xb[i].type);
#ifdef Layer1m11      /* print only [1,-1,1] layer (rgbt t=transparency) */
      if( (r1[0]-r1[1]+r1[2])*(r1[0]-r1[1]+r1[2])>0.8
       || (r3[0]-r3[1]+r3[2])*(r3[0]-r3[1]+r3[2])>0.8) farb="T";
#endif
      printf("cone {\n <% 6.3f,% 6.3f,% 6.3f>,0.5\n <% 6.3f,% 6.3f,% 6.3f>,0.5\n pigment { color %s%d }\n finish {ambient 0.2 diffuse 0.8 }\n no_shadow }\n",
                zoom*r1[0], zoom*r1[1], zoom*r1[2],
                zoom*r2[0], zoom*r2[1], zoom*r2[2], farb, j%4 );
      printf("cone {\n <% 6.3f,% 6.3f,% 6.3f>,0.5\n <% 6.3f,% 6.3f,% 6.3f>,0.5\n pigment { color %s%d }\n finish {ambient 0.2 diffuse 0.8 }\n no_shadow }\n",
                zoom*r3[0], zoom*r3[1], zoom*r3[2],
                zoom*r4[0], zoom*r4[1], zoom*r4[2], farb, j%4 );
    }
    printf("// END of model\n");
    return 0;
} /* out_pov */

/* -------------------------------------------------------------- */
/* ToDo19: add bond filter bitmask like m_1d.c */
int main(int argn, char *argv[]){
  int i=1, j, v, nlat, lat=-1, typ=0, vvv=1;
  for (nlat=0;uclist[nlat].xn;nlat++);
  while (argn>i && argv[i][0]=='-' && argv[i][1]>='a') {
    if (argv[i][1]=='l'){
       if (argv[i][2]>='0' && argv[i][2]<='9') {
         lat=atoi(argv[i]+2);
         if (lat>=0 && lat<nlat) { i++; continue; } /* valid? */
       } /* -l0 .. -l7 */
       else { /* -l$name p.e. -l fcc */
         for (j=0;uclist[j].xn;j++)
           if (strstr(uclist[j].xn,argv[i]+2)) { lat=j; break; }
       } /* -l<argument name|number> */
       if (lat<0 || lat >= nlat) {
         printf(" try option -l<n> where n is one of the list below\n");
         for (j=0;uclist[j].xn;j++)
           printf(" %2d: %s\n", j, uclist[j].xn);
         return 1; /* list only */
       } // bad lat-number
    }
    if (argv[i][1]=='t') typ=atoi(argv[i]+2);
    if (argv[i][1]=='d') dim=atoi(argv[i]+2);
    if (argv[i][1]=='a') vvv=0;
    i++;
  }
  if (!vvv) xa=til11_xa; /* use simple cubic (orthonormal) base */
  if (lat==-1) { /* [-][0-9] <= 'a' */
    fprintf(stderr,"no valid lattice choosen (option -l<n>)\n"
     " options: -l[<n>] [-t<0..2>] [-a] [-d<dim>] [e00 e01 ... [... e22]]\n");
    // 2019-09-11 add description of options -a, -t, stderr (old=stdout)
    fprintf(stderr," -t<0..2>  0=def-file 1=povray-file 2=gnuplot-3d-file\n");
    fprintf(stderr," -a        use cubic base, ignore base definition\n");
    for (i=0; i<32 && uclist[i].xn!=NULL ;i++)
      fprintf(stderr,"  -l%d %s\n", i, uclist[i].xn);
    return 2;
  }
  /* ToDo: dim==0 for finite lattice, remove bonds to other unitcells? */
  if (argn== i+1 || argn== i+4 || argn== i+9) {
    dim=0;
    if( argn-i == 1) dim=1;
    if( argn-i == 4) dim=2;
    if( argn-i == 9) dim=3;
    for(j=0;j<dim*dim;j++)xe[j/dim][j%dim]=atoi(argv[j+i]);
  } else { /* bad or no edge-vectors, ToDo: dim from definitions */
    dim= 0;
    for (i=0; uclist[lat].xb[i].type ;i++) { /* auto detect dim from UC-bonds */
      if (dim<1 && uclist[lat].xb[i].r2[0] != 0) dim=1;
      if (dim<2 && uclist[lat].xb[i].r2[1] != 0) dim=2;
      if (dim<3 && uclist[lat].xb[i].r2[2] != 0) dim=3;
    } /* for i */
    fprintf(stderr,"DBG: missing arguments (edgevectors, example: 1 0  0 1)\n");
    fprintf(stderr,"DBG; autodetected dimendsion D=%d\n", dim);
//    return 3;
  } /* edgevec arguments */
  title="unknown";
  if (lat>=0 && lat<nlat) {
    title=uclist[lat].xn;
    if (vvv) xa = uclist[lat].xa;;
    xs = uclist[lat].xs;;
    xb = uclist[lat].xb;;
  } else {
    fprintf(stderr,"ERR: bad lattice number n=%d (0..%d)\n", lat, nlat-1);
    return 3;
  }
  
  // fprintf(stderr,"DBG_%d: til=%d title=%s dim=%d\n",__LINE__, lat, title, dim);
  nn=v=get_deti(xe);         /* volume of cluster */

  if( v<=0 ){ fprintf(stderr,"error: volume<=0\n");return -1; }
  vertex_init();
  ini_y( xa, xe, xs, xb );  /* malloc ys + yb */
  
  if(typ==0){   out_def( lat, xa, xe, ys, yb ); }
  if(typ==1){   out_pov( lat, xa, xe, ys, yb ); }
  if(typ==2){ out_splot( lat, xa, xe, ys, yb ); }

  if (ys) free((void *)ys);
  if (yb) free((void *)yb);
  return 0;  
}

